@@ -1,10 +1,10 @@
 -- KeyInfo.hs: OpenPGP (RFC4880) fingerprinting methods
--- Copyright © 2012-2013  Clint Adams
+-- Copyright © 2012-2014  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
 module Codec.Encryption.OpenPGP.KeyInfo (
-   keySize
+   pubkeySize
  , pkalgoAbbrev
 ) where
 
@@ -18,9 +18,10 @@ import Data.List (unfoldr)
 
 import Codec.Encryption.OpenPGP.Types
 
-keySize (RSAPubKey x) = RSA.public_size x * 8
-keySize (DSAPubKey x) = bitcount . DSA.params_p . DSA.public_params $ x
-keySize (ElGamalPubKey x) = bitcount $ head x
+pubkeySize (RSAPubKey x) = Right (RSA.public_size x * 8)
+pubkeySize (DSAPubKey x) = Right (bitcount . DSA.params_p . DSA.public_params $ x)
+pubkeySize (ElGamalPubKey x) = Right (bitcount $ head x)
+pubkeySize x = Left $ "Unable to calculate size of " ++ show x
 
 bitcount = (*8) . length . unfoldr (\x -> if x == 0 then Nothing else Just (True, x `shiftR` 8))
 
