@@ -181,6 +181,8 @@ getSigSubPacket = do
                        return $ SigSubPacket crit (TrustSignature tl ta)
             | pt == 6 = do
                        apdre <- getByteString (l - 2)
+		       nul <- getWord8
+		       guard (nul == 0)
                        return $ SigSubPacket crit (RegularExpression (B.copy apdre))
             | pt == 7 = do
                        r <- get
@@ -193,9 +195,10 @@ getSigSubPacket = do
                        return $ SigSubPacket crit (PreferredSymmetricAlgorithms sa)
             | pt == 12 = do
                        rclass <- getWord8
+                       guard (testBit rclass 7)
                        algid <- get
                        fp <- getByteString 20
-                       return $ SigSubPacket crit (RevocationKey (bsToFFSet . B.singleton $ rclass) algid (TwentyOctetFingerprint fp))
+                       return $ SigSubPacket crit (RevocationKey (bsToFFSet . B.singleton $ rclass .&. 0x7f) algid (TwentyOctetFingerprint fp))
             | pt == 16 = do
                        keyid <- getByteString (l - 1)
                        return $ SigSubPacket crit (Issuer (EightOctetKeyId keyid))
@@ -290,7 +293,7 @@ putSigSubPacket (SigSubPacket crit (PreferredSymmetricAlgorithms ess)) = do
 putSigSubPacket (SigSubPacket crit (RevocationKey rclass algid fp)) = do
     putSubPacketLength 23
     putSigSubPacketType crit 12
-    putByteString . ffSetToFixedLengthBS 1 $ rclass
+    putByteString . ffSetToFixedLengthBS 1 $ Set.insert (RClOther 0) rclass
     put algid
     putByteString (unTOF fp) -- 20 octets
 putSigSubPacket (SigSubPacket crit (Issuer keyid)) = do
@@ -496,12 +499,12 @@ getPacketTypeAndPayload = do
                        3 -> do len <- remaining
                                bs <- getByteString len
                                return (t, bs)
-                       _ -> error "This should never happen."
+                       _ -> error "This should never happen (getPacketTypeAndPayload/0x00)."
         0x40 -> do
                    len <- fmap fromIntegral getPacketLength
                    bs <- getByteString len
                    return (tag .&. 0x3f, bs)
-        _ -> error "This should never happen."
+        _ -> error "This should never happen (getPacketTypeAndPayload/???)."
 
 getPkt :: Get Pkt
 getPkt = do
