@@ -3,24 +3,27 @@
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
-{-# LANGUAGE DeriveDataTypeable, ExistentialQuantification, TemplateHaskell, TypeFamilies #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric, ExistentialQuantification, TemplateHaskell, TypeFamilies #-}
 
 module Codec.Encryption.OpenPGP.Types where
 
-import qualified Crypto.PubKey.RSA as RSA
-import qualified Crypto.PubKey.DSA as DSA
+import GHC.Generics (Generic)
 
 import Control.Arrow ((***))
 import Control.Lens (makeLenses)
+import qualified Crypto.PubKey.RSA as RSA
+import qualified Crypto.PubKey.DSA as DSA
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
 import Data.Char (toLower, toUpper)
 import Data.Data (Data)
+import Data.Hashable (Hashable(..))
 import Data.IxSet (IxSet)
 import Data.List.Split (chunksOf)
 import Data.Monoid ((<>))
 import Data.Ord (comparing)
 import Data.Set (Set)
+import qualified Data.Set as Set
 import Data.Typeable (Typeable)
 import Data.Word (Word8, Word16, Word32)
 import Numeric (readHex, showHex)
@@ -59,7 +62,7 @@ data SymmetricAlgorithm = Plaintext
                         | AES256
                         | Twofish
                         | OtherSA Word8
-     deriving (Data, Show, Typeable)
+     deriving (Data, Generic, Show, Typeable)
 
 instance Eq SymmetricAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -93,9 +96,11 @@ instance FutureVal SymmetricAlgorithm where
     toFVal 10 = Twofish
     toFVal o = OtherSA o
 
+instance Hashable SymmetricAlgorithm
+
 data NotationFlag = HumanReadable
-                  | OtherNF Int
-     deriving (Data, Show, Typeable)
+                  | OtherNF Word8 -- FIXME: this should be constrained to 4 bits?
+     deriving (Data, Generic, Show, Typeable)
 
 instance Eq NotationFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -110,14 +115,18 @@ instance FutureFlag NotationFlag where
     toFFlag 0 = HumanReadable
     toFFlag o = OtherNF (fromIntegral o)
 
+instance Hashable NotationFlag
+
 data SigSubPacket = SigSubPacket {
     _sspCriticality :: Bool
   , _sspPayload :: SigSubPacketPayload
-  } deriving (Data, Eq, Typeable)
+  } deriving (Data, Eq, Generic, Typeable)
 
 instance Show SigSubPacket where
     show x = (if _sspCriticality x then "*" else "") ++ (show . _sspPayload) x
 
+instance Hashable SigSubPacket
+
 data SigSubPacketPayload = SigCreationTime TimeStamp
                   | SigExpirationTime TimeStamp
                   | ExportableCertification Exportability
@@ -143,7 +152,9 @@ data SigSubPacketPayload = SigCreationTime TimeStamp
                   | EmbeddedSignature SignaturePayload
                   | UserDefinedSigSub Word8 ByteString
                   | OtherSigSub Word8 ByteString
-    deriving (Data, Eq, Show, Typeable) -- FIXME
+    deriving (Data, Eq, Generic, Show, Typeable) -- FIXME
+
+instance Hashable SigSubPacketPayload
 
 data HashAlgorithm = DeprecatedMD5
                    | SHA1
@@ -153,7 +164,7 @@ data HashAlgorithm = DeprecatedMD5
                    | SHA512
                    | SHA224
                    | OtherHA Word8
-    deriving (Data, Show, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq HashAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -179,12 +190,14 @@ instance FutureVal HashAlgorithm where
     toFVal 11 = SHA224
     toFVal o = OtherHA o
 
+instance Hashable HashAlgorithm
+
 data CompressionAlgorithm = Uncompressed
                           | ZIP
                           | ZLIB
                           | BZip2
                           | OtherCA Word8
-    deriving (Show, Data, Typeable)
+    deriving (Show, Data, Generic, Typeable)
 
 instance Eq CompressionAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -204,6 +217,8 @@ instance FutureVal CompressionAlgorithm where
     toFVal 3 = BZip2
     toFVal o = OtherCA o
 
+instance Hashable CompressionAlgorithm
+
 class (Eq a, Ord a) => FutureVal a where
    fromFVal :: a -> Word8
    toFVal :: Word8 -> a
@@ -218,7 +233,7 @@ data PubKeyAlgorithm = RSA
                      | ForbiddenElgamal
                      | DH
                      | OtherPKA Word8
-    deriving (Show, Data, Typeable)
+    deriving (Show, Data, Generic, Typeable)
 
 instance Eq PubKeyAlgorithm where
     (==) a b = fromFVal a == fromFVal b
@@ -248,13 +263,15 @@ instance FutureVal PubKeyAlgorithm where
     toFVal 21 = DH
     toFVal o = OtherPKA o
 
+instance Hashable PubKeyAlgorithm
+
 class (Eq a, Ord a) => FutureFlag a where
     fromFFlag :: a -> Int
     toFFlag :: Int -> a
 
 data KSPFlag = NoModify
              | KSPOther Int
-    deriving (Data, Show, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq KSPFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -269,6 +286,8 @@ instance FutureFlag KSPFlag where
     toFFlag 0 = NoModify
     toFFlag i = KSPOther (fromIntegral i)
 
+instance Hashable KSPFlag
+
 data KeyFlag = GroupKey
              | AuthKey
              | SplitKey
@@ -277,7 +296,7 @@ data KeyFlag = GroupKey
              | SignDataKey
              | CertifyKeysKey
              | KFOther Int
-    deriving (Data, Show, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq KeyFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -304,9 +323,11 @@ instance FutureFlag KeyFlag where
     toFFlag 7 = CertifyKeysKey
     toFFlag i = KFOther (fromIntegral i)
 
+instance Hashable KeyFlag
+
 data RevocationClass = SensitiveRK
-                     | RClOther Int
-    deriving (Data, Show, Typeable)
+                     | RClOther Word8 -- FIXME: this should be constrained to 3 bits
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq RevocationClass where
     (==) a b = fromFFlag a == fromFFlag b
@@ -321,13 +342,15 @@ instance FutureFlag RevocationClass where
     toFFlag 1 = SensitiveRK
     toFFlag i = RClOther (fromIntegral i)
 
+instance Hashable RevocationClass
+
 data RevocationCode = NoReason
                     | KeySuperseded
                     | KeyMaterialCompromised
                     | KeyRetiredAndNoLongerUsed
                     | UserIdInfoNoLongerValid
                     | RCoOther Word8
-    deriving (Show, Data, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq RevocationCode where
     (==) a b = fromFVal a == fromFVal b
@@ -349,9 +372,11 @@ instance FutureVal RevocationCode where
     toFVal 32 = UserIdInfoNoLongerValid
     toFVal o = RCoOther o
 
+instance Hashable RevocationCode
+
 data FeatureFlag = ModificationDetection
                  | FeatureOther Int
-    deriving (Show, Data, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq FeatureFlag where
     (==) a b = fromFFlag a == fromFFlag b
@@ -366,16 +391,26 @@ instance FutureFlag FeatureFlag where
     toFFlag 7 = ModificationDetection
     toFFlag i = FeatureOther (fromIntegral i)
 
+instance Hashable FeatureFlag
+instance Hashable a => Hashable (Set a) where
+    hashWithSalt salt = hashWithSalt salt . Set.toList
+
 newtype MPI = MPI {unMPI :: Integer}
-    deriving (Data, Eq, Show, Typeable)
+    deriving (Data, Eq, Generic, Show, Typeable)
+
+instance Hashable MPI
 
 data SignaturePayload = SigV3 SigType Word32 EightOctetKeyId PubKeyAlgorithm HashAlgorithm Word16 [MPI]
                       | SigV4 SigType PubKeyAlgorithm HashAlgorithm [SigSubPacket] [SigSubPacket] Word16 [MPI]
                       | SigVOther Word8 ByteString
-    deriving (Data, Eq, Show, Typeable) -- FIXME
+    deriving (Data, Eq, Generic, Show, Typeable) -- FIXME
+
+instance Hashable SignaturePayload
 
 data KeyVersion = DeprecatedV3 | V4
-    deriving (Data, Eq, Ord, Show, Typeable)
+    deriving (Data, Eq, Generic, Ord, Show, Typeable)
+
+instance Hashable KeyVersion
 
 data PKPayload = PKPayload {
       _keyVersion :: KeyVersion
@@ -383,22 +418,31 @@ data PKPayload = PKPayload {
     , _v3exp :: V3Expiration
     , _pkalgo :: PubKeyAlgorithm
     , _pubkey :: PKey
-    } deriving (Data, Eq, Show, Typeable)
+    } deriving (Data, Eq, Generic, Show, Typeable)
 
 instance Ord PKPayload where
-    compare a b = show a `compare` show b -- FIXME: this is ridiculous
+    compare = comparing _keyVersion <> comparing _timestamp <> comparing _v3exp <> comparing _pkalgo <> comparing (show . _pubkey) -- FIXME: this is suboptimal
+
+instance Hashable PKPayload
 
 data SKAddendum = SUS16bit SymmetricAlgorithm S2K IV ByteString
                 | SUSSHA1 SymmetricAlgorithm S2K IV ByteString
                 | SUSym SymmetricAlgorithm IV ByteString
                 | SUUnencrypted SKey Word16
-    deriving (Data, Eq, Show, Typeable)
+    deriving (Data, Eq, Generic, Show, Typeable)
+
+instance Ord SKAddendum where
+    compare a b = show a `compare` show b -- FIXME: this is ridiculous
+
+instance Hashable SKAddendum
 
 data DataType = BinaryData
               | TextData
               | UTF8Data
               | OtherData Word8
-    deriving (Show, Data, Typeable)
+    deriving (Show, Data, Generic, Typeable)
+
+instance Hashable DataType
 
 instance Eq DataType where
     (==) a b = fromFVal a == fromFVal b
@@ -421,18 +465,33 @@ data S2K = Simple HashAlgorithm
          | Salted HashAlgorithm Salt
          | IteratedSalted HashAlgorithm Salt Count
          | OtherS2K Word8 ByteString
-    deriving (Data, Eq, Show, Typeable) -- FIXME
+    deriving (Data, Eq, Generic, Show, Typeable) -- FIXME
+
+instance Hashable S2K
 
 data UserAttrSubPacket = ImageAttribute ImageHeader ImageData
                        | OtherUASub Word8 ByteString
-    deriving (Data, Eq, Show, Typeable) -- FIXME
+    deriving (Data, Eq, Generic, Show, Typeable) -- FIXME
+
+instance Hashable UserAttrSubPacket
+
+instance Ord UserAttrSubPacket where
+    compare (ImageAttribute h1 d1) (ImageAttribute h2 d2) = compare h1 h2 <> compare d1 d2
+    compare (ImageAttribute _ _) (OtherUASub _ _) = LT
+    compare (OtherUASub _ _) (ImageAttribute _ _) = GT
+    compare (OtherUASub t1 b1) (OtherUASub t2 b2) = compare t1 t2 <> compare b1 b2
 
 data ImageHeader = ImageHV1 ImageFormat
-    deriving (Data, Eq, Show, Typeable)
+    deriving (Data, Eq, Generic, Show, Typeable)
+
+instance Ord ImageHeader where
+    compare (ImageHV1 a) (ImageHV1 b) = compare a b
+
+instance Hashable ImageHeader
 
 data ImageFormat = JPEG
                  | OtherImage Word8
-    deriving (Data, Show, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq ImageFormat where
     (==) a b = fromFVal a == fromFVal b
@@ -447,6 +506,8 @@ instance FutureVal ImageFormat where
     toFVal 1 = JPEG
     toFVal o = OtherImage o
 
+instance Hashable ImageFormat
+
 data SigType = BinarySig
              | CanonicalTextSig
              | StandaloneSig
@@ -463,7 +524,7 @@ data SigType = BinarySig
              | TimestampSig
              | ThirdPartyConfirmationSig
              | OtherSig Word8
-    deriving (Data, Show, Typeable)
+    deriving (Data, Generic, Show, Typeable)
 
 instance Eq SigType where
     (==) a b = fromFVal a == fromFVal b
@@ -506,26 +567,45 @@ instance FutureVal SigType where
     toFVal 0x50 = ThirdPartyConfirmationSig
     toFVal o = OtherSig o
 
+instance Hashable SigType
+
 instance Ord DSA.PublicKey
 instance Ord RSA.PublicKey
+instance Ord DSA.PrivateKey
+instance Ord RSA.PrivateKey
+
+instance Hashable DSA.Params where
+    hashWithSalt s (DSA.Params p g q) = s `hashWithSalt` p `hashWithSalt` g `hashWithSalt` q
+instance Hashable DSA.PublicKey where
+    hashWithSalt s (DSA.PublicKey p y) = s `hashWithSalt` p `hashWithSalt` y
+instance Hashable DSA.PrivateKey where
+    hashWithSalt s (DSA.PrivateKey p x) = s `hashWithSalt` p `hashWithSalt` x
+instance Hashable RSA.PublicKey where
+    hashWithSalt s (RSA.PublicKey size n e) = s `hashWithSalt` size `hashWithSalt` n `hashWithSalt` e
+instance Hashable RSA.PrivateKey where
+    hashWithSalt s (RSA.PrivateKey pub d p q dP dQ qinv) = s `hashWithSalt` pub `hashWithSalt` d `hashWithSalt` p `hashWithSalt` q `hashWithSalt` dP `hashWithSalt` dQ `hashWithSalt` qinv
 
 data PKey = RSAPubKey RSA.PublicKey
           | DSAPubKey DSA.PublicKey
           | ElGamalPubKey [Integer]
 	  | UnknownPKey ByteString
-    deriving (Data, Eq, Ord, Show, Typeable)
+    deriving (Data, Eq, Generic, Ord, Show, Typeable)
+
+instance Hashable PKey
 
 data SKey = RSAPrivateKey RSA.PrivateKey
           | DSAPrivateKey DSA.PrivateKey
           | ElGamalPrivateKey [Integer]
 	  | UnknownSKey ByteString
-    deriving (Data, Eq, Show, Typeable)
+    deriving (Data, Eq, Generic, Show, Typeable)
+
+instance Hashable SKey
 
 newtype Block a = Block {unBlock :: [a]} -- so we can override cereal instance
     deriving (Show, Eq)
 
 newtype EightOctetKeyId = EightOctetKeyId {unEOKI :: ByteString}
-    deriving (Eq, Ord, Data, Typeable) -- FIXME
+    deriving (Eq, Ord, Data, Generic, Typeable) -- FIXME
 
 instance Show EightOctetKeyId where
     show = w8sToHex . B.unpack . unEOKI
@@ -533,8 +613,10 @@ instance Show EightOctetKeyId where
 instance Read EightOctetKeyId where
     readsPrec _ = map ((EightOctetKeyId . B.pack *** concat) . unzip) . chunksOf 8 . hexToW8s
 
+instance Hashable EightOctetKeyId
+
 newtype TwentyOctetFingerprint = TwentyOctetFingerprint {unTOF :: ByteString}
-    deriving (Eq, Ord, Data, Typeable)
+    deriving (Data, Eq, Generic, Ord, Typeable)
 
 instance Show TwentyOctetFingerprint where
     show = take 50 . concatMap (++" ") . concatMap (++[""]) . chunksOf 5 . chunksOf 4 . w8sToHex . B.unpack . unTOF
@@ -542,6 +624,8 @@ instance Show TwentyOctetFingerprint where
 instance Read TwentyOctetFingerprint where
     readsPrec _ = map ((TwentyOctetFingerprint . B.pack *** concat) . unzip) . chunksOf 20 . hexToW8s . filter (/= ' ')
 
+instance Hashable TwentyOctetFingerprint
+
 w8sToHex :: [Word8] -> String
 w8sToHex = map toUpper . concatMap ((\x -> if length x == 1 then '0':x else x) . flip showHex "")
 
@@ -549,16 +633,15 @@ hexToW8s :: ReadS Word8
 hexToW8s = concatMap readHex . chunksOf 2 . map toLower
 
 data TK = TK {
-    _tkPKP :: PKPayload
-  , _tkmSKA :: Maybe SKAddendum
+    _tkKey  :: (PKPayload, Maybe SKAddendum)
   , _tkRevs :: [SignaturePayload]
   , _tkUIDs :: [(String, [SignaturePayload])]
   , _tkUAts :: [([UserAttrSubPacket], [SignaturePayload])]
-  , _tkSubs :: [(Pkt, SignaturePayload, Maybe SignaturePayload)]
+  , _tkSubs :: [(Pkt, [SignaturePayload])]
   } deriving (Data, Eq, Show, Typeable)
 
 instance Ord TK where
-    compare = comparing _tkPKP -- FIXME: is this ridiculous?
+    compare = comparing _tkKey -- FIXME: is this ridiculous?
 
 type Keyring = IxSet TK
 
@@ -589,7 +672,9 @@ data Pkt = PKESKPkt PacketVersion EightOctetKeyId PubKeyAlgorithm [MPI]
          | ModificationDetectionCodePkt ByteString
          | OtherPacketPkt Word8 ByteString
          | BrokenPacketPkt String Word8 ByteString
-    deriving (Data, Eq, Show, Typeable) -- FIXME
+    deriving (Data, Eq, Generic, Show, Typeable) -- FIXME
+
+instance Hashable Pkt
 
 pktTag :: Pkt -> Word8
 pktTag (PKESKPkt {}) = 1
